#!/usr/bin/python
# -*- coding: UTF-8 -*- 
#
# Gdesklet Control to assist in presenting an image slide show
#
# Author: Sean Bridges (sean@mailcan.com)
#
# Revision History
# 04-JUN-07		First Release
# 11-JUN-07		Single Image mode
#				More Error Detection
# 17-JUN-07		Threaded the directory walk, hoping to make it more responsive.
# 03-SEP-07     Misc Bug Fixes, Error Handling
# TODO
#	Make storage of thumbnails smart so we don't overlap and have fade
#	

from libdesklets.controls import Control
from PIL import Image
from IImageSlideShow import IImageSlideShow
import iptc, EXIF, jpegcom, directorywalk, random, sys, re, thread, os.path, commands

#
# Provide functions needed for a slide show
#
class ImageSlideShow(Control, IImageSlideShow):


	def __init__(self):
		debugger("Called: __init__" )
		# Default values
		self.__default_image_directory= ""
		self.__image_directory = self.__default_image_directory
		self.__default_image_list = [ "" ]
		self.__image_list = self.__default_image_list
		self.__image_list_length = 0
		self.__image_counter = 0 
		self.__random_images = True
		self.__image_timer = 10
		self.__image_timer_id = 0
		self.__good_directory = False
		self.__thumbnail_size = (300,225)
		self.__thumbnail_prefix = "A"
		self.__thumbnail_instance = str(random.randrange(0, 10000, 2))
		self.__previous_thumb = ""
		self.__previous_width = 300
		
		#Cache folder
		#folder = os.path.expanduser("~") + "/" + ".cache2/imageslideshow"
		folder = "/tmp/imageslideshow"
		debugger("cache folder is:" + folder)
		if not os.path.isdir(folder):
			os.mkdir(folder)
			
		self.__thumbnail_folder=folder
		
		Control.__init__(self)

		
	def __get_next_image(self):
		debugger("Called: __get_next_image")
		self.__image_counter = self.__image_counter + 1
		#Reset counter if over limits
		if self.__image_counter >= self.__image_list_length: self.__image_counter=0
		self._update("image_uri") #Trigger binds
		self._update("thumbnail_uri") #Trigger binds
		self.__set_image_timer( self.__image_timer ) #Restart Timer
		return self.__get_image()

	def __get_prev_image(self): 
		debugger("Called: __get_prev_image")	
		self.__image_counter = self.__image_counter - 1
		if self.__image_counter < 0: self.__image_counter=self.__image_list_length-1
		self._update("image_uri")
		self._update("thumbnail_uri") #Trigger binds
		self.__set_image_timer( self.__image_timer ) #Restart Timer
		return self.__get_image()

	def __get_image(self): 
		debugger("Called:__get_image. Returned" + self.__image_list[self.__image_counter] )
		return self.__image_list[self.__image_counter]

	def __get_thumbnail(self): 
		debugger("Called:__get_thumbnail. Using " + self.__get_image() )
		debugger("Called:__get_thumbnail. Size " + str(self.__thumbnail_size) )

		destination=self.__thumbnail_folder + "/" + self.__thumbnail_prefix + self.__thumbnail_instance + "-thumbnail.png"

		if (self.__previous_thumb != self.__get_image() or self.__previous_width != self.__thumbnail_size[0] ):
			try:
				if (self.__thumbnail_prefix == "B"):
					self.__thumbnail_prefix = "A"
				else:
					self.__thumbnail_prefix = "B"
					
				destination=self.__thumbnail_folder + "/" + self.__thumbnail_prefix + self.__thumbnail_instance + "-thumbnail.png"
				thumb = Image.open(self.__get_image())
				thumb.thumbnail(self.__thumbnail_size, Image.ANTIALIAS)
				thumb.save(destination)
				self.__previous_thumb = self.__get_image()
				self.__previous_width = self.__thumbnail_size[0]
				
			except:
				print "Unexpected error:", sys.exc_info()[0]
				pass
			
		#return self.__image_list[self.__image_counter]
		return destination
		
	def __set_image(self,image_uri): 
		debugger("Called:__set_image")	
		self.__image_list_length = 1
		self.__image_counter = 0 		
		self.__image_list[:]=[ image_uri ]
		self.__image_directory=""
		self._update("image_uri")
		self._update("thumbnail_uri")

	def __get_iptc_meta_data(self): 
		debugger("Called:__get_iptc_meta_data")		
		try:
			return iptc.getiptc_path(self.__get_image())
		except:
			return {}

	def __get_exif_meta_data(self): 
		debugger("Called:__get_exif_meta_data")		
		try:
			f=open(self.__get_image(), 'rb')
			return EXIF.process_file(f)
		except:
			print "Unexpected error:", sys.exc_info()[0]
			return {}

	def __get_image_caption(self): 
		debugger("Called:__get_image_caption")	
		#Make sure file exists
		method=""
		caption=""
		caption_return=""		
		if os.path.isfile ( self.__get_image() ):

			try:
				meta_dic = self.__get_iptc_meta_data()
				#Strip Nulls from null terminated strings
				caption=str(meta_dic['Caption'].strip('\x00'))
				if caption != "": method="IPTC: "
			except:
				pass
			
			#See if there is a JPEG Comment
			if caption == "":
				try:
					#Strip Nulls from null terminated strings
					# Picasa adds these.
					caption=str(jpegcom.read_comments(self.__get_image()))
					if caption != "": method="JPEGCOM: "
				except:
					pass			

			#See if there is an EXIF description if there is no IPTC or JPEG COM
			if caption == "":
				try:
					tags=self.__get_exif_meta_data()
					caption=str(tags['ImageDescription'])
					if caption != "": method="EXIF: "
				except:
					pass		
				
			#return method + caption
			try:
				caption=caption.encode('utf_8','replace') 
				caption=re.sub('<[^!>](?:[^>]|\n)*>', '', caption)
			except:
				caption=""

		return caption

		
	def __get_directory(self): 
		debugger("Called:__get_directory")	
		return self.__image_directory

	def __set_directory_threaded(self, directory):
		debugger("Called:__set_directory_threaded:" + directory)	
		if os.path.isdir ( directory ):
			thread.start_new_thread(self.__set_directory, ( directory, ))

	def __set_directory(self, directory):	
		debugger("Called:__set_directory:" + directory)	
		if directory != self.__default_image_directory:
			try:
				image_list=directorywalk.make_directory_list(directory)	
				image_list_length=len(image_list)
				
				for i in range(image_list_length):
					print image_list[i]
				print "Total number is " + str(len(image_list))
				
				if image_list_length > 0:
					self.__image_directory = directory
					self.__image_list_length=image_list_length
					self.__image_counter=0 
					if self.__random_images: random.shuffle(image_list)
					self.__image_list[:]=image_list
					self._update("image_uri")
					self._update("thumbnail_uri") #Trigger binds
			except:
				#reset to defaults if we fail
				#print "Setting Defaults because of" , sys.exc_info()[0]
				self.__image_directory=self.__default_image_directory
				self.__image_list[:] = self.__default_image_list
		
	def __set_random_images(self, random ): 
		debugger("Called: __get_image")		
		oldrandom=self.__random_images
		self.__random_images=random
		if oldrandom != random:
			self.__set_directory(self.__image_directory)

	def __get_random_images(self):
		debugger("Called:__get_random_images")	
		return self.__random_images

	def __set_image_timer(self, seconds ): 
		debugger("Called:__set_image_timer")		
		self.__image_timer=seconds
		try:
			#Remove current timer
			self._remove_timer(self.__image_timer_id)
		except:
			pass
		if seconds > 0:
			self.__image_timer_id = self._add_timer(seconds*1000,self.__get_next_image)

	def __get_image_timer(self): 
		debugger("Called: __get_image")	
		return self.__image_timer

	def __get_image_refresh(self): 
		debugger("Called:__get_image_refresh")	
		if( self.__get_image != "" ):
			self._update("image_uri") #Trigger binds
			self._update("thumbnail_uri") #Trigger binds
		return True

	def __get_thumbnail_size(self):
		debugger("Called: __get_image_size")	
		return self.__thumbnail_size
		
	def __set_thumbnail_size(self,size):
		debugger("Called: __set_image_size")
		self.__thumbnail_size=size
		
	def __launch(self,command):
		import os
		os.system(command + " &")
		
	def _shutdown(self):
		#Cleanup any thumbnail caches
		debugger("Called:_shutdown")
		pattern = re.compile('.+thumbnail\.png')
		try:
			for each in os.listdir(self.__thumbnail_folder):
				if pattern.search(each):
					name = os.path.join(self.__thumbnail_folder, each)
					debugger("REMOVING: " + name)
					os.remove(name)
		except:
			raise
			
	#
	# Properties
	#
	image_uri       = property( fget = __get_image, 
	                            fset = __set_image, 
	                            doc="Get or Set the current image uri. This property is bindable")
	iptc_meta_data  = property( fget = __get_iptc_meta_data,
	                            doc="Get RAW IPTC dictionary")
	exif_meta_data  = property( fget = __get_exif_meta_data,
	                            doc="Get RAW EXIF dictionary")
	image_caption   = property( fget = __get_image_caption,
	                            doc="Get caption string")
	image_directory = property( fset = __set_directory,
	                            fget=__get_directory, 
	                            doc="Set or Get the picture directory")
	next_image      = property( fget = __get_next_image,
	                            doc="Get the URI of the next image")
	prev_image      = property( fget =__get_prev_image,
	                            doc="Get the URI of the prev image")
	random_images   = property( fset = __set_random_images,
	                            fget = __get_random_images,
	                            doc="Get or Set Bool.  Will set the control to send random images or not")
	image_timer     = property( fset = __set_image_timer,
	                            fget = __get_image_timer,
	                            doc="Enables or disables a timer on the next_image method. Pass a time in seconds, and then bind to the next_image property")
	image_refresh   = property( fget = __get_image_refresh,
	                            doc="This causes an update to be made on the image_uri property. Forcing any bound functions to be called")
	thumbnail_size  = property( fget = __get_thumbnail_size,
	                            fset = __set_thumbnail_size,
	                            doc  = "Sets the default size of the thumbnail, string '600x600'")
	thumbnail_uri   = property( fget = __get_thumbnail, 
	                            doc="Get the uri for the generated thumbnail")
	launch          = property( fset = __launch,
	                            doc="Launch an external program for the current image")
	                            
def get_class(): return ImageSlideShow

def debugger(text):
	if False:
		print "ImageSlideShow " + text
